/*
   Copyright (C) 2000-2001  Ulric Eriksson <ulric@siag.nu>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston,
   MA 02111-1307, USA.
 */

#include "config.h"

#ifdef HAVE_LIBGDBM

/*
skey=value stores the key key with the value value
fkey fetches the value of the key key
dkey deletes the key key
l lists all keys
r reorganizes the database
ekey tests if key key exists

The url looks like gdbm:db=database
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <gdbm.h>
#include "common.h"
#include "sdb.h"

#define CLOSEDB(dbf) { gdbm_close(dbf); dbf = NULL; }

static GDBM_FILE dbf;

static datum s2d(char *s)
{
	datum d;
	d.dptr = s;
	d.dsize = strlen(s)+1;
	return d;
}

#if 0	/* unused */
static char *d2s(datum d)
{
	return d.dptr;
}
#endif

static void die(char *p)
{
#if 0
printf("die(%s)\n", p);
#endif
	fprintf(stderr, "Cause of death: '%s'\n", p);
	if (dbf) gdbm_close(dbf);
}

static GDBM_FILE openordie(char *name, int rw)
{
	GDBM_FILE dbf = gdbm_open(name, 512, rw, 0644, die);
#if 0
printf("openordie(%s, %d) => %p\n", name, rw, dbf);
#endif
	if (dbf == NULL) die("Can't open database");
	return dbf;
}

static int gdbm_driver(char *dd, char *q,
	int (*callback)(int, char **, void *), void *closure)
{
	char *database = sdb_url_value(dd, "db");

	int res = 0;
	int r;
	char *k, *v;
	datum d;

	if (database == NULL) {
printf("no database\n");
		return -1;
	}

	switch (q[0]) {
	case 's':
		k = q+1;
		v = strchr(k, '=');
		if (v == NULL) die(q);
		*v++ = '\0';
		dbf = openordie(database, GDBM_WRCREAT);
		r = gdbm_store(dbf, s2d(k), s2d(v), GDBM_REPLACE);
		if (r == -1) die(q);
		CLOSEDB(dbf);
		break;
	case 'f':
		k = q+1;
		dbf = openordie(database, GDBM_READER);
		d = gdbm_fetch(dbf, s2d(k));
		v = d.dptr;
		if (v) (*callback)(1, &v, closure);
		else res = -1;
		free(v);
		CLOSEDB(dbf);
		break;
	case 'd':
		k = q+1;
		dbf = openordie(database, GDBM_WRCREAT);
		r = gdbm_delete(dbf, s2d(k));
		CLOSEDB(dbf);
		if (r) {
#if 0
fprintf(stderr, "Can't delete '%s'\n", k);
#endif
			res = -1;
		} else {
#if 0
fprintf(stderr, "Key '%s' deleted\n", k);
#endif
			res = 0;
		}
		break;
	case 'l':
#if 0
		printf("------\n");
#endif
		dbf = openordie(database, GDBM_READER);
		d = gdbm_firstkey(dbf);
		while (d.dptr) {
			k = d.dptr;
			(*callback)(1, &k, closure);
#if 0
printf("%s\n", k);
#endif
			d = gdbm_nextkey(dbf, d);
			free(k);
		}
		CLOSEDB(dbf);
#if 0
printf("------\n");
#endif
		break;
	case 'r':
		dbf = openordie(database, GDBM_WRCREAT);
		gdbm_reorganize(dbf);
		CLOSEDB(dbf);
fprintf(stderr, "Database reorganized\n");
		break;
	case 'e':
		k = q+1;
		dbf = openordie(database, GDBM_READER);
		r = gdbm_exists(dbf, s2d(k));
		if (r) {
#if 0
fprintf(stderr, "Exists\n");
#endif
			res = 0;
		} else {
#if 0
fprintf(stderr, "Doesn't exist\n");
#endif
			res = -1;
		}
		CLOSEDB(dbf);
		break;
	default:
		die(q);
		break;
	}
	if (dbf) gdbm_close(dbf);
	/* end of code from slow.c */

	sdb_free(database);
	return res;
}

void sdb_init_gdbm(void)
{
	sdb_register_driver("gdbm", gdbm_driver);
}

#else

void sdb_init_gdbm(void)
{
	;
}

#endif	/* GDBM */
