/*
   Copyright (C) 2000-2001  Ulric Eriksson <ulric@siag.nu>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the Licence, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with this library; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston,
   MA 02111-1307, USA.
 */
 
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <sys/time.h>
#include <unistd.h>
#include "config.h"
#include "common.h"

extern void sdb_init_gdbm(void);	    /* gdbm.c */
extern void sdb_init_lago(void);            /* lago.c */
extern void sdb_init_mysql(void);           /* mysql.c */
extern void sdb_init_sqlite(void);          /* sqlite.c */
extern void sdb_init_mimer(void);           /* mimer.c */
extern void sdb_init_postgres(void);        /* postgres.c */
extern void sdb_init_oracle(void);          /* oracle.c */
extern void sdb_init_odbc(void);		/* odbc.c */

static struct sdb_driver {
	char *name;
	int (*driver)(char *, char *, int (*)(int, char **, void *), void *);
} drivers[20];

static int ndrivers;

char *sdb_url_value(char *p, char *k)
{
        int len = strlen(k);
        char b[1024];
        int i;
 
        while (p) {
                if (!sdb_strncasecmp(p, k, len) && p[len] == '=') {
                        p = p+len+1;
                        for (i = 0; p[i] && p[i] != ':'; i++)
                                b[i] = p[i];
                        b[i] = '\0';
                        return sdb_strdup(b);
                }
                p = strchr(p, ':');
                if (p) p++;
        }
        return NULL;
}

void sdb_register_driver(char *name,
	int (*driver)(char *, char *, int (*)(int, char **, void *), void *))
{
	drivers[ndrivers].name = name;
	drivers[ndrivers].driver = driver;
	ndrivers++;
}

/*
;@db_query(url, query)
;@Makes database queries. <a href="db.html">More information</a>.
;@db_query("odbc:postgresql", "select * from foo")
;@
*/
int sdb_query(char *url, char *query,
	int (*callback)(int, char **, void *), void *closure)
{
        char *u, *p;
        int i;
 
        if (url == NULL || query == NULL) return -1;
 
        u = sdb_strdup(url);
        p = strchr(u, ':');
        if (p == NULL) return -1;
 
        *p++ = '\0';
 
        for (i = 0; drivers[i].name; i++) {
                if (!sdb_strcasecmp(drivers[i].name, u)) {
                        return (*drivers[i].driver)(p, query, callback, closure);
                }
        }
	return -1;
}


void sdb_init(void)
{
	ndrivers = 0;
        sdb_init_lago();
        sdb_init_mysql();
        sdb_init_sqlite();
        sdb_init_mimer();
        sdb_init_postgres();
        sdb_init_oracle();
	sdb_init_odbc();
	sdb_init_gdbm();
}
